<?php

namespace Drupal\entity_template_ui\Controller;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Builder\ConfigurableDefaultBlueprintInterface;
use Drupal\entity_template\TemplateBuilderManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class TemplateBuilderDefaultBlueprintController extends ControllerBase {

  /**
   * @var \Drupal\entity_template\TemplateBuilderManager
   */
  protected $builderManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_template.builder'),
      $container->get('entity_type.manager'),
      $container->get('entity.form_builder')
    );
  }

  /**
   * TemplateBuilderDefaultBlueprintController constructor.
   *
   * @param \Drupal\entity_template\TemplateBuilderManager $builder_manager
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   */
  public function __construct(
    TemplateBuilderManager $builder_manager,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFormBuilderInterface $entity_form_builder
  ) {
    $this->builderManager = $builder_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFormBuilder = $entity_form_builder;
  }

  /**
   * Get the edit default blueprint form for this builder.
   *
   * @param string $builder
   *   The name of the builder plugin.
   *
   * @return array
   *   The page content.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function editDefaultBlueprint(string $builder) {
    try {
      $plugin = $this->builderManager->createInstance($builder);

      if (!$plugin instanceof ConfigurableDefaultBlueprintInterface) {
        throw new PluginException('Builder '.$builder.' does not implement '.ConfigurableDefaultBlueprintInterface::class);
      }
    }
    catch (PluginException $exception) {
      throw new NotFoundHttpException('No builder exists with that name', $exception);
    }

    $blueprint = $plugin->getDefaultBlueprintConfig();
    $operation = "default";
    if ($this->entityTypeManager->getDefinition('entity_template_blueprint')->getFormClass('edit')) {
      $operation = 'edit';
    }
    return $this->entityFormBuilder->getForm($blueprint, $operation);
  }

  /**
   * Get the title of the edit page for the default blueprint.
   *
   * @param string $builder
   *   The builder plugin id.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The page title.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function editDefaultBlueprintTitle(string $builder) {
    $definition = $this->builderManager->getDefinition($builder);
    return $this->t('Edit @label Default Blueprint', ['@label' => $definition['label']]);
  }
}
