<?php

namespace Drupal\entity_template_ui\Plugin\EntityTemplate\Template;

use Drupal\Core\Url;

class BlueprintTemplateUI extends BaseTemplateUI {

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\BlueprintTemplateInterface
   */
  protected $template;

  /**
   * {@inheritdoc}
   */
  public function addComponentUrl(): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.choose_component',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_id' => $this->template->getKey(),
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function swapComponentUrl(string $key, string $other_id): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.swap_component',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
        'component_id' => $key,
        'other_plugin' => $other_id,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function removeComponentUrl(string $key): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.remove_component',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
        'component_id' => $key,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function addConditionUrl(): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.choose_condition',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function configureConditionUrl(string $key): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.configure_condition',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
        'condition_id' => $key,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function removeConditionUrl(string $key): ?Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template.remove_condition',
      [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
        'condition_id' => $key,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function dataSelectAutocompleteElement(string $component_id): array {
    return [
      '#autocomplete_route_name' => 'entity_template_ui.edit_template.component.dsautocomplete',
      '#autocomplete_route_parameters' => [
        'template_blueprint_provider' => $this->template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $this->template->getBlueprint()->getProviderKey(),
        'template_key' => $this->template->getKey(),
        'component_id' => $component_id,
      ],
    ];
  }
}
