<?php

namespace Drupal\entity_template_ui\Plugin\EntityTemplate\Template;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;

class TemplateUIFactory implements TemplateUIFactoryInterface {

  /**
   * @var \Drupal\Core\DependencyInjection\ClassResolverInterface
   */
  protected $classResolver;

  /**
   * TemplateUIFactory constructor.
   *
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   */
  public function __construct(ClassResolverInterface $class_resolver) {
    $this->classResolver = $class_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function ui(TemplateInterface $template): TemplateUIInterface {
    $definition = $template->getPluginDefinition();
    $class = isset($definition['ui_class']) ? $definition['ui_class'] : BlueprintTemplateUI::class;

    $ui = $this->classResolver->getInstanceFromDefinition($class);

    if (!$ui || !($ui instanceof TemplateUIInterface)) {
      throw new InvalidPluginDefinitionException(
        $template->getPluginId(),
        "Invalid ui class specified."
      );
    }

    return $ui->setTemplate($template);
  }

}
