<?php

namespace Drupal\entity_template;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;

interface BlueprintStorageInterface extends BlueprintInterface {

  /**
   * Get the blueprint provider for this blueprint storage.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface
   */
  public function getProvider();

  /**
   * Get the key that identifies this in the provider.
   *
   * @return string
   */
  public function getProviderKey();

  /**
   * Get the tempstore key.
   *
   * @return string
   */
  public function getTempstoreKey();

  /**
   * Add a template to the collection.
   *
   * @param array $configuration
   *
   * @return string
   *   The id of the template.
   */
  public function addTemplate(array $configuration);

  /**
   * Remove a template by id.
   *
   * @param $id
   */
  public function removeTemplate($id);

  /**
   * Get the edit url for this blueprint.
   *
   * @return \Drupal\Core\Url
   */
  public function getEditUrl();

  /**
   * Get the edit template url.
   *
   * @param string $key
   *
   * @return \Drupal\Core\Url
   */
  public function getEditTemplateUrl(string $key): Url;

  /**
   * Test the access for this blueprint storage.
   *
   * @param string $operation
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param boolean $return_as_object
   *
   * @return \Drupal\Core\Access\AccessResultInterface|bool
   */
  public function access($operation, AccountInterface $account, $return_as_object = TRUE);
}
