<?php

namespace Drupal\entity_template\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\entity_template\Blueprint;
use Drupal\entity_template\BlueprintInterface;
use Drupal\entity_template\BlueprintEntityAdaptor;

/**
 * Class TemplateBlueprint
 *
 * @ConfigEntityType(
 *   id = "entity_template_blueprint",
 *   label = @Translation("Template Blueprint"),
 *   admin_permission = "administer template blueprints",
 *   config_prefix = "blueprint",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "builder",
 *     "is_default",
 *     "templates",
 *   },
 * )
 *
 * @package Drupal\entity_template\Entity
 */
class TemplateBlueprint extends ConfigEntityBase implements BlueprintEntityInterface {

  /**
   * @var string
   */
  protected $id;

  /**
   * @var string
   */
  protected $label;

  /**
   * @var string
   */
  protected $builder;

  /**
   * We use this config entities for default as well as override blueprints. This
   * flag marks whether the blueprint is a default.
   *
   * @var bool
   */
  protected $is_default = FALSE;

  /**
   * @var array
   */
  protected $templates = [];

  /**
   * {@inheritdoc}
   */
  public function toBlueprint(): BlueprintInterface {
    return new BlueprintEntityAdaptor($this);
  }

  /**
   * {@inheritdoc}
   */
  public function updateFromBlueprint(BlueprintInterface $blueprint): BlueprintEntityInterface {
    $this->builder = $blueprint->getBuilder()->getPluginId();
    $this->templates = $blueprint->getTemplates()->getConfiguration();

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getBuilder(): string {
    return $this->get('builder');
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplates(): array {
    return $this->get('templates');
  }

}
