<?php

namespace Drupal\entity_template;

use Drupal\Core\TempStore\PrivateTempStoreFactory;

/**
 * Tempstore repository for entity builder sessions.
 */
class EntityBuilderTempStoreRepository implements EntityBuilderTempStoreRepositoryInterface {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * EntityBuilderTempStoreRepository.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory) {
    $this->tempStoreFactory = $temp_store_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getTempstoreId() {
    return 'entity_builder_session';
  }

  /**
   * {@inheritdoc}
   */
  public function getTempstore() {
    return $this->tempStoreFactory->get($this->getTempstoreId());
  }

  /**
   * {@inheritdoc}
   */
  public function getBuilderResult(string $session_key) {
    return $this->getTempstore()->get('builder_result__' . $session_key);
  }

  /**
   * {@inheritdoc}
   */
  public function setBuilderResult(string $session_key, BlueprintResult $result) {
    $this->getTempstore()->set('builder_result__' . $session_key, $result);
  }

}
