<?php

namespace Drupal\entity_template\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ConfigTemplateBuilderDeriver
 *
 * @package Drupal\entity_template\Plugin\Derivative
 */
class ConfigTemplateBuilderDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * @var string
   */
  protected $basePluginId;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $base_plugin_id,
      $container->get('entity_type.manager')
    );
  }

  /**
   * ConfigTemplateBuilderDeriver constructor.
   *
   * @param $base_plugin_id
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct($base_plugin_id, EntityTypeManagerInterface $entity_type_manager) {
    $this->basePluginId = $base_plugin_id;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    /** @var \Drupal\entity_template\Entity\TemplateBuilder $builder */
    foreach ($this->entityTypeManager->getStorage('entity_template_builder')->loadMultiple() as $builder) {
      $context_definitions = [];
      foreach ($builder->getParameters() as $parameter) {
        if (strpos($parameter['type'], 'entity:') === 0) {
          $context_definitions[$parameter['machine_name']] = new EntityContextDefinition($parameter['type'], $parameter['label']);
        } else {
          $context_definitions[$parameter['machine_name']] = new ContextDefinition(
            $parameter['type'],
            $parameter['label']
          );
        }
      }

      $this->derivatives[$builder->id()] = [
          'template_builder' => $builder->id(),
          'label' => $builder->label(),
          'context_definitions' => $context_definitions,
        ] + $base_plugin_definition;
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }
}
