<?php

namespace Drupal\entity_template\Plugin\TypedDataFilter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\ListDataDefinitionInterface;
use Drupal\typed_data\DataFilterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A data filter for formatting fields.
 *
 * @DataFilter(
 *   id = "format_field",
 *   label = @Translation("Formats a field using a specified formatter."),
 * )
 */
class FieldFormatterFilter extends DataFilterBase implements ContainerFactoryPluginInterface {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return (new static($configuration, $plugin_id, $plugin_definition))
      ->setRenderer($container->get('renderer'));
  }

  /**
   * Set the renderer service.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   *
   * @return $this
   */
  public function setRenderer(RendererInterface $renderer) : FieldFormatterFilter {
    $this->renderer = $renderer;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function filter(
    DataDefinitionInterface $definition,
    $value,
    array $arguments,
    BubbleableMetadata $bubbleable_metadata = NULL
  ) {
    assert($value instanceof FieldItemListInterface);
    $build = $value->view([
      'type' => reset($arguments),
      'label' => 'hidden',
    ]);
    return Markup::create($this->renderer->renderRoot($build));
  }

  /**
   * {@inheritdoc}
   */
  public function canFilter(DataDefinitionInterface $definition) {
    return $definition instanceof FieldItemDataDefinitionInterface
      || ($definition instanceof ListDataDefinitionInterface && $definition->getItemDefinition() instanceof FieldItemDataDefinitionInterface);
  }

  /**
   * {@inheritdoc}
   */
  public function usesWrappedValue() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function filtersTo(DataDefinitionInterface $definition, array $arguments) {
    return DataDefinition::create('markup');
  }
}
