<?php

namespace Drupal\entity_template\Plugin\TypedDataFilter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\ListDataDefinitionInterface;
use Drupal\Core\TypedData\OptionsProviderInterface;
use Drupal\options\Plugin\Field\FieldType\ListItemBase;
use Drupal\typed_data\DataFilterBase;

/**
 * A data filter for formatting fields.
 *
 * @DataFilter(
 *   id = "option_label",
 *   label = @Translation("Converts a selected value into its readable label (Options fields)."),
 * )
 */
class OptionLabelFilter extends DataFilterBase {

  /**
   * {@inheritdoc}
   */
  public function filter(
    DataDefinitionInterface $definition,
    $value,
    array $arguments,
    BubbleableMetadata $bubbleable_metadata = NULL
  ) {
    $first_item = $value instanceof FieldItemListInterface ?
      $value->get(0) : $value;

    $allowed_values = [];
    if ($first_item instanceof OptionsProviderInterface) {
      $allowed_values = $first_item->getPossibleOptions();
    }

    $mapped = [];
    if ($value instanceof FieldItemListInterface) {
      foreach ($value as $item) {
        $mapped[] = $allowed_values[$item->value] ?? $item->value;
      }
    }
    else if ($value instanceof ListItemBase) {
      $mapped[] = $allowed_values[$value->value] ?? $value->value;
    }

    return implode(', ', $mapped);
  }

  /**
   * {@inheritdoc}
   */
  public function canFilter(DataDefinitionInterface $definition) {
    $definition = ($definition instanceof ListDataDefinitionInterface) ? $definition->getItemDefinition() : $definition;

    return $definition instanceof FieldItemDataDefinitionInterface && is_subclass_of($definition->getClass(), ListItemBase::class);
  }

  /**
   * {@inheritdoc}
   */
  public function filtersTo(DataDefinitionInterface $definition, array $arguments) {
    return DataDefinition::create('string');
  }

  /**
   * {@inheritdoc}
   */
  public function usesWrappedValue() {
    return TRUE;
  }
}
