<?php

namespace Drupal\entity_template;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

class TemplateComponentManager extends DefaultPluginManager {

  /**
   * TemplateBuilderManager constructor.
   *
   * @param \Traversable $namespaces
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct(
      'Plugin/EntityTemplate/Component',
      $namespaces,
      $module_handler, 'Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface',
      'Drupal\entity_template\Annotation\EntityTemplateComponent'
    );
    $this->alterInfo('entity_template_component');
    $this->setCacheBackend($cache_backend, 'entity_template_component');
  }

  /**
   * Get applicable definitions.
   *
   * @param string $return_type
   */
  public function getApplicableDefinitions($return_type) {
    $bits = explode(':', $return_type);
    do {
      $return_types[] = implode(':', $bits);
      array_pop($bits);
    } while (count($bits));

    return array_filter($this->getDefinitions(), function ($definition) use ($return_types) {
      return array_intersect($definition['applies_to'], $return_types);
    });
  }

}
